#pragma once
#include <glm/glm.hpp>
#include <cstdint>

#include "types.hpp"

//All packages need to be smaller than 512, otherwise the package needs to be fragmented.
//In case of an fragmented package, all parts of it are gathered and sorted based on the information provide in the packet.
//Currently only PacketFrameNal, PacketFrameMetadata packages could get fragmented during transmission.
//Besides that the transport system has to wait for the PacketSetup, PacketSetupComplete to arrive before it can distribute other packages.
//This means that the application always sees the PacketSetup, PacketSetupComplete before any other package.

//NOTE: Not an actual packet.
struct PacketHeader
{
    PacketId id = (PacketId)0;
    uint32_t size = 0;
};

struct PacketSetup
{
    PacketId id = PACKET_ID_SETUP;
    uint32_t size = 0;

    glm::u32vec2 resolution;
};

struct PacketSetupComplete
{
    PacketId id = PACKET_ID_SETUP_COMPLETE;
    uint32_t size = 0;

    PacketStereoStrategyId strategy_id;
    uint32_t max_frame_ids;

    float near_plane;
    float far_plane;
};

struct PacketProjectionChange
{
    PacketId id = PACKET_ID_PROJECTION_CHANGE;
    uint32_t size = 0;

    glm::mat4 left_eye_projection;
    glm::mat4 right_eye_projection;

    glm::mat4 left_head_to_eye;
    glm::mat4 right_head_to_eye;
};

struct PacketStageTransform
{
    PacketId id = PACKET_ID_STAGE_TRANSFORM;
    uint32_t size = 0;

    glm::mat4 stage_transform;
};

struct PacketHeadTransform
{
    PacketId id = PACKET_ID_HEAD_TRANSFORM;
    uint32_t size = 0;

    PacketTransformId transform_id;
    glm::mat4 head_transform;
};

struct PacketControllerTransform
{
    PacketId id = PACKET_ID_CONTROLLER_TRANSFORM;
    uint32_t size = 0;

    PacketControllerId controller_id;
    glm::mat4 controller_transform;
};

struct PacketControllerEvent
{
    PacketId id = PACKET_ID_CONTROLLER_EVENT;
    uint32_t size = 0;

    PacketControllerId controller_id;
    PacketControllerState controller_state;
};

struct PacketControllerButton
{
    PacketId id = PACKET_ID_CONTROLLER_TRANSFORM;
    uint32_t size = 0;

    PacketControllerId controller_id;
    PacketButtonId button_id;
    PacketButtonState button_state;
};

struct PacketControllerThumbstick
{
    PacketId id = PACKET_ID_CONTROLLER_THUMBSTICK;
    uint32_t size = 0;

    PacketControllerId controller_id;
    glm::vec2 thumbstick;
};

//NOTE: The entire size of the packet including the payload needs to be less than 512 bytes
struct PacketFrameConfigNal
{
    PacketId id = PACKET_ID_FRAME_NAL;
    uint32_t size = 0;

    PacketFrameId frame_id;

    //Followed by: Config Nal
};

//NOTE: The packet needs to be fragmented if the payload is too big
struct PacketFrameNal
{
    PacketId id = PACKET_ID_FRAME_NAL;
    uint32_t size = 0;

    PacketFrameNumber frame_number; //In case of fragmentation, frame number and frame id are used together as identifier
    PacketFrameId frame_id;
    PacketTransformId transform_id;

    uint32_t payload_offset;
    uint32_t payload_size;

    //Followed by: Nal
};

//NOTE: Assume frame metadata of a particular frame number is always send before any frame nals of the same frame number
//NOTE: The packet needs to be fragmented if the payload is too big
struct PacketFrameMetadata
{
    PacketId id = PACKET_ID_FRAME_METADATA;
    uint32_t size = 0;

    PacketFrameNumber frame_number; //In case of fragmentation, the frame number is used as identifier

    uint32_t payload_offset;
    uint32_t payload_size;

    //Followed by: Metadata
};

struct PacketLatency
{
    PacketId id = PACKET_ID_LATENCY;
    uint32_t size = 0;

    PacketFrameNumber frame_number;
    PacketFrameId frame_id;
    PacketTransformId transform_id;

    double timestamp_transform_query = 0.0;
    double timestamp_server_response = 0.0;
    double timestamp_frame_decoded = 0.0;
    double timestamp_command_construct = 0.0;
    double timestamp_command_executed = 0.0;
};